@@ -9,7 +9,7 @@ import numpy as np
 import yaml
 import os
 import json
-from logger import logger
+from datacenter_utils.logger import logger
 
 
 def read_yaml(file_path, key='pg'):
@@ -51,12 +51,101 @@ class Postgres(object):
 
     def is_table_exist(self, table_str):
         # TODO 表名字符长度限制63
-        exec_str = f"select exists(select * from information_schema.tables where table_name='{table_str.lower()[:63]}');"
+        exec_str = "select exists(select * from information_schema.tables where table_name='{}');".format(
+            table_str.lower()[:63])
         self.cursor.execute(exec_str)
         exist = self.cursor.fetchone()[0]
-        logger.info(f'{table_str} is_table_exist: {exist}')
+        logger.info('{} is_table_exist: {}'.format(table_str, exist))
         return exist
 
+    def upsert_df(self, df, table_name, constraint_columns,
+                  date_columns=None, timestamp_columns=None
+                  ):
+        """
+        dataframe按照规定的字段设计upsert到指定数据库。
+        针对constraint_columns列表，df任一记录任一字段与数据库不一样，就会插入一条新的数据，否则会更新数据库里原有记录。
+
+        :param df: 待插入到数据库里的dataframe
+        :type df: pandas.Dataframe
+        :param table_name: 目标表名
+        :type table_name: str
+        :param constraint_columns: 表结构设计中的primary key
+        :type constraint_columns: list of str
+        :param date_columns: 定义为date类型的字段
+        :type date_columns: list of str
+        :param timestamp_columns:  定义为timestamp类型的字段
+        :type timestamp_columns: list of str
+        """
+        data_list = df.values.tolist()
+        columns = list(df.columns)
+        column_list = ['"{}"'.format(c) for c in columns]
+        exclude_columns = column_list.copy()
+        constraint_columns = ['"{}"'.format(i) for i in constraint_columns]
+
+        for remove_c in constraint_columns:
+            exclude_columns.remove(remove_c)
+        for row in data_list:
+            if len(column_list) - len(constraint_columns) == 1:
+                sql = '''
+                           INSERT INTO {t} 
+                                VALUES  ({column_str})
+                                ON CONFLICT ({constraint_columns})
+                                DO UPDATE SET
+                                    {exclude_columns}
+                                    = {exclude_values} ;
+                        '''
+            elif len(column_list) - len(constraint_columns) > 1:
+                sql = '''
+                           INSERT INTO {t} 
+                                VALUES  ({column_str})
+                                ON CONFLICT ({constraint_columns})
+                                DO UPDATE SET
+                                    ({exclude_columns})
+                                    = ({exclude_values}) ;
+                        '''
+            else:
+                raise Exception('length of constraint_columns error')
+
+            for i in range(len(row)):
+                if date_columns and column_list[i] in date_columns:
+                    if isinstance(row[i], pd.Timestamp):
+                        row[i] = row[i].date()
+                    elif isinstance(row[i], str):
+                        row[i] = datetime.strptime(row[i], '%Y-%m-%d').date()
+                    elif isinstance(row[i], date):
+                        pass
+                    elif isinstance(row[i], datetime):
+                        row[i] = row[i].date()
+                    else:
+                        row[i] = None
+
+                    if timestamp_columns and column_list[i] in timestamp_columns:
+                        if isinstance(row[i], pd.Timestamp):
+                            row[i] = row[i].datetime()
+                        elif isinstance(row[i], str):
+                            row[i] = datetime.strptime(row[i], '%Y-%m-%d %H:%M:%S')
+                        elif isinstance(row[i], date):
+                            pass
+                        elif isinstance(row[i], datetime):
+                            pass
+                        else:
+                            row[i] = None
+                if row[i] == '' or (isinstance(row[i], float) and np.isnan(row[i])):
+                    row[i] = None
+
+            c_str = ','.join(['%s' for i in range(len(row))])
+
+            exclude_values = ['EXCLUDED.{}'.format(i) for i in exclude_columns]
+            sql_format = sql.format(t=table_name,
+                                    column_str=c_str,
+                                    constraint_columns=','.join(constraint_columns),
+                                    exclude_columns=','.join(exclude_columns),
+                                    exclude_values=','.join(exclude_values))
+            self.cursor.execute(sql_format, row)
+
+        logger.info('saved {}'.format(table_name))
+        self.database_postgres.commit()
+
     def update_insert_df(self, df, table_name, text_columns, constraint_columns,
                          date_columns=None, timestamp_columns=None
                          ):
@@ -64,7 +153,7 @@ class Postgres(object):
         dataframe按照规定的字段设计upsert到指定数据库。
         针对constraint_columns列表，df任一记录任一字段与数据库不一样，就会插入一条新的数据，否则会更新数据库里原有记录。
         当表（table_name）不存在时，该函数操作会自动创建表，按照字段类型参数（text_columns、constraint_columns、date_columns、
-        timestamp_columns）创建相应类型，未指定的按照text类型处理
+        timestamp_columns）创建相应类型，未指定的按照float类型处理
 
         :param df: 待插入到数据库里的dataframe，其columns需要明确且与text_columns、constraint_columns、date_columns、timestamp_columns保持一致
         :type df: pandas.Dataframe
@@ -86,7 +175,6 @@ class Postgres(object):
             text_columns.extend(date_columns)
         if timestamp_columns:
             text_columns.extend(timestamp_columns)
-        # self.cursor.execute(f'DROP TABLE IF EXISTS "{table_name}";')
         count = 0
         column_list = []
         c_str = ''
@@ -97,18 +185,18 @@ class Postgres(object):
 
                 if not text_columns:
                     text_columns = column_list
-                sql_str = ','.join([f'"{j.replace(" ", "_")}" float' if j not in text_columns
-                                    else f'"{j.replace(" ", "_")}" text'
+                sql_str = ','.join(['"{}" float'.format(j.replace(" ", "_")) if j not in text_columns
+                                    else '"{}" text'.format(j.replace(" ", "_"))
                                     for j in column_list])
                 if not date_columns:
                     date_columns = []
                 for dd in date_columns:
-                    sql_str = sql_str.replace(f'"{dd}" text', f'"{dd}" date')
+                    sql_str = sql_str.replace('"{}" text'.format(dd), '"{}" date'.format(dd))
 
                 if not timestamp_columns:
                     timestamp_columns = []
                 for dd in timestamp_columns:
-                    sql_str = sql_str.replace(f'"{dd}" text', f'"{dd}" timestamp')
+                    sql_str = sql_str.replace('"{}" text'.format(dd), '"{}" timestamp'.format(dd))
 
                 # sql_s = re.findall(r'[^\*/:?\\|<>!]', sql_s, re.S)
                 # sql_s = "".join(sql_s)
@@ -118,9 +206,9 @@ class Postgres(object):
                     add_conflict_sql = '''
                     ALTER TABLE "public"."{t}" ADD CONSTRAINT "{pkey}" PRIMARY KEY ({keys});
                     '''.format(t=table_name, pkey=table_name + '_pkey',
-                               keys=','.join([f'"{i}"' for i in constraint_columns])
+                               keys=','.join(['"{}"'.format(i) for i in constraint_columns])
                                )
-                    self.cursor.execute(f'create table {table_name} ({sql_str});')
+                    self.cursor.execute('create table {} ({});'.format(table_name, sql_str))
                     self.cursor.execute(add_conflict_sql)
                     logger.info('create table', sql_str)
             else:
@@ -183,9 +271,9 @@ class Postgres(object):
                     exclude_list = column_list.copy()
                     for remove_c in constraint_columns:
                         exclude_list.remove(remove_c)
-                    exclude_columns = [f'"{i}"' for i in exclude_list]
-                    constraint_columns = [f'"{i}"' for i in constraint_columns]
-                    exclude_values = [f'EXCLUDED."{i}"' for i in exclude_list]
+                    exclude_columns = ['"{}"'.format(i) for i in exclude_list]
+                    constraint_columns = ['"{}"'.format(i) for i in constraint_columns]
+                    exclude_values = ['EXCLUDED."{}"'.format(i) for i in exclude_list]
                     sql_str = sql_str.replace('text', '').replace('date', '').replace('float', '')
                 sql_format = sql.format(t=table_name,
                                         column_str=c_str,
@@ -196,7 +284,7 @@ class Postgres(object):
                 self.cursor.execute(sql_format, row)
             count += 1
 
-        logger.info(f'saved {table_name}')
+        logger.info('saved {}'.format(table_name))
         self.database_postgres.commit()
 
     def find(self, table, filter_dict=None, columns=None):
@@ -212,7 +300,7 @@ class Postgres(object):
         :return: 查询结果
         """
         if columns:
-            columns_str = ",".join([f'"{i}"' for i in columns])
+            columns_str = ",".join(['"{}"'.format(i) for i in columns])
         else:
             columns_str = '*'
         filter_list = []
@@ -220,12 +308,12 @@ class Postgres(object):
             filter_dict = {}
         for k, v in filter_dict.items():
             if isinstance(v, str):
-                v = f"'{v}'"
+                v = "'{}'".format(v)
             elif isinstance(v, (datetime, date)):
-                v = f"'{v}'"
-            filter_list.append(f'"{k}" = {v}')
+                v = "'{}'".format(v)
+            filter_list.append('"{}" = {}'.format(k, v))
         filter_str = 'where' + ' and '.join(filter_list) if filter_list else ''
-        sql = f"select {columns_str} from {table} {filter_str}"
+        sql = "select {} from {} {}".format(columns_str, table, filter_str)
         logger.info('sql:', sql)
         self.cursor.execute(sql)
         res = self.cursor.fetchall()
@@ -244,7 +332,7 @@ class Postgres(object):
         :return: 查询结果
         """
         if columns:
-            columns_str = ",".join([f'"{i}"' for i in columns])
+            columns_str = ",".join(['"{}"'.format(i) for i in columns])
         else:
             columns_str = '*'
         filter_list = []
@@ -252,12 +340,12 @@ class Postgres(object):
             filter_dict = {}
         for k, v in filter_dict.items():
             if isinstance(v, str):
-                v = f"'{v}'"
+                v = "'{}'".format(v)
             elif isinstance(v, (datetime, date)):
-                v = f"'{v}'"
-            filter_list.append(f'"{k}" = {v}')
+                v = "'{}'".format(v)
+            filter_list.append('"{}" = {}'.format(k, v))
         filter_str = 'where' + ' and '.join(filter_list) if filter_list else ''
-        sql = f"select {columns_str} from {table} {filter_str}"
+        sql = "select {} from {} {}".format(columns_str, table, filter_str)
         logger.info('sql:', sql)
         self.cursor.execute(sql)
         res = self.cursor.fetchone()
@@ -286,3 +374,19 @@ class Postgres(object):
         # 指定字符去除
         line2 = re.sub(r"[‘’“”，。？?]", "", line1)
         return line2
+
+
+if __name__ == '__main__':
+    config = dict(database="test",
+                  user="read_write",
+                  password="read_write",
+                  host="10.50.1.236",
+                  port="5432")
+    p = Postgres(conf=config)
+    df = pd.DataFrame({"date": [datetime(2021, 1, 1), datetime(2021, 1, 2)],
+                       "factor_ret": [1.4444, 1.23],
+                       "factor_rankIC": [1.22, 1.23],
+                       "group3_ret": [1.22, 1.23],
+                       "group0_ret": [1.22, 1.23],
+                       })
+    p.upsert_df(df, "stock_hs300_longshort_pred", ['date'], ['date'])
